
clear
set seed 15
capture cd "C:\Users\ChuckStata\Google Drive\Stata\blog\2018\powersim1\examples"
capture cd "C:\Users\jch\Google Drive\Stata\blog\2018\powersim1\examples"

capture log close
log using powersim1.log, text replace

global GraphWidth16x9 = 1920
global GraphHeight16x9 = 1080

// SCALARS
scalar i = 1
display "i = " i

scalar i = 1
display "i = " scalar(i)

// LOCALS
local i = 1
display "i = `i'"


// GENERATE PSEUDO-RANDOM DATASETS
clear
set seed 15
set obs 200
generate randnum= runiform()
list randnum in 1/5

generate age = runiformint(18,65)
list age in 1/5

generate female = rbinomial(1,0.5)
list female in 1/5

generate weight = rnormal(72,15)
generate height = rnormal(170,10)
list weight height in 1/5

webuse nhanes2, clear
summ weight height
correlate height weight


matrix m = (72,170)
matrix s = (15,10)
matrix C = (1.0, 0.5 \ 0.5, 1.0)
drawnorm weight height, n(200) means(m) sds(s) corr(C) clear double
summarize
correlate
		

		

// STORE MODEL COEFFICIENTS TO LOCAL MACROS
ttest weight = 70
return list
scalar pvalue = r(p)
display "The two-sided p-value is", pvalue



// SIMPLE PROGRAM TO SHOW INPUT AND OUTPUT
capture program drop myprogram
program myprogram, rclass
	version 15
	syntax, n(integer)
	display "n = `n'"
	return scalar N = `n'
end

myprogram, n(50)
return list


capture program drop simttest
program simttest, rclass
	version 15.1
	// DEFINE THE INPUT PARAMETERS AND THEIR DEFAULT VALUES
							
	// GENERATE THE RANDOM DATA AND TEST THE NULL HYPOTHESIS						
	
	// RETURN RESULTS
end


// BASIC T-TEST SIMULATION
capture program drop simttest
program simttest, rclass
    version 15.1 
	
    // DEFINE THE INPUT PARAMETERS AND THEIR DEFAULT VALUES
    syntax, n(integer)          ///  Sample size
          [ alpha(real 0.05)    ///  Alpha level
            m0(real 0)          ///  Mean under the null
            ma(real 1)          ///  Mean under the alternative
            sd(real 1) ]        //   Standard deviation

    // GENERATE THE RANDOM DATA AND TEST THE NULL HYPOTHESIS		  
    drawnorm y, n(`n') means(`ma') sds(`sd') clear  
    ttest y = `m0' 
	
    // RETURN RESULTS
    return scalar reject = (r(p)<`alpha') 
end

simttest, n(100) m0(70) ma(75) sd(15) alpha(0.05)

return list

// USE -simulate- TO RUN -simttest- 100 TIMES
simulate reject=r(reject), reps(100) seed(12345): 	///
	simttest, n(100) m0(70) ma(75) sd(15) alpha(0.05)

list in 1/5
summarize reject
local power = r(mean)
display "Power = `power'"


// COMPUTE POWER USING -power onemean-
power onemean 70 75, n(100) sd(15) alpha(0.05)




log close



