
clear
set seed 12345
capture cd "C:\Users\ChuckStata\Google Drive\Stata\blog\2018\powersim1b\examples"
capture cd "C:\Users\jch\Google Drive\Stata\blog\2018\powersim1b\examples"

capture log close
log using powersim1b.log, text replace

global GraphWidth16x9 = 1200
global GraphHeight16x9 = 720


// COMPUTE POWER USING -power onemean- FOR A 
// RANGE OF SAMPLE SIZES AND GRAPH THE RESULTS
power onemean 70 75, n(50(10)100) sd(15) alpha(0.05)

power onemean 70 75, n(50(10)100) sd(15) alpha(0.05) graph
graph export ./graphs/power_onemean.png, as(png) width($GraphWidth16x9) height($GraphHeight16x9) replace


// BASIC T-TEST SIMULATION (DO NOT INCLUDE IN BLOG 2)
capture program drop simttest
program simttest, rclass
	version 15.1
	syntax,   n(integer)			/// 
			[ alpha(real 0.05) 		///
			  m0(real 0) 			///
			  ma(real 1) 			///
			  sd(real 1) ]
	drawnorm y, n(`n') means(`ma') sds(`sd') clear double 
	quietly ttest y = `m0' 
	return scalar reject = (r(p)<`alpha') 
end

simulate reject=r(reject), reps(100) seed(12345): 	///
	simttest, n(100) m0(70) ma(75) sd(15) alpha(0.05)

summarize reject
local power = r(mean)
display "power = `power'"




capture program drop power_cmd_simttest
program power_cmd_simttest, rclass
	version 15.1
	
	// DEFINE THE INPUT PARAMETERS AND THEIR DEFAULT VALUES
    syntax, n(integer)          ///  Sample size
          [ alpha(real 0.05)    ///  Alpha level
            m0(real 0)          ///  Mean under the null
            ma(real 1)          ///  Mean under the alternative
            sd(real 1)          ///  Standard deviation
            reps(integer 100)]  //   Number of repetitions
			  
	// GENERATE THE RANDOM DATA AND TEST THE NULL HYPOTHESIS
	quietly simulate reject=r(reject), reps(`reps'): 	///
			simttest, n(`n') m0(`m0') ma(`ma') sd(`sd') alpha(`alpha')
	quietly summarize reject
	
	// RETURN RESULTS
	return scalar power = r(mean)
	return scalar N = `n'
	return scalar alpha = `alpha'
	return scalar m0 = `m0'
	return scalar ma = `ma'
	return scalar sd = `sd'
end

power simttest, n(100) m0(70) ma(75) sd(15)
power simttest, n(50(10)100) m0(70) ma(75) sd(15) table graph
graph export ./graphs/simttest1.png, as(png) width($GraphWidth16x9) height($GraphHeight16x9) replace




capture program drop power_cmd_simttest_init
program power_cmd_simttest_init, sclass
	// ADD COLUMNS TO THE OUTPUT TABLE
	sreturn local pss_colnames "m0 ma sd"
	// ALLOW NUMLISTS FOR m0, ma, and sd
	sreturn local pss_numopts  "m0 ma sd"
end

power simttest, n(75 100) m0(70) ma(72(1)75) sd(15) reps(1000)
power simttest, n(75 100) m0(70) ma(72(1)75) sd(15) reps(1000) graph(xdimension(ma))
graph export ./graphs/simttest2.png, as(png) width($GraphWidth16x9) height($GraphHeight16x9) replace




