

clear
set seed 15
capture cd "C:\Users\ChuckStata\Google Drive\Stata\blog\2018\powersim2\examples"
capture cd "C:\Users\jch\Google Drive\Stata\blog\2018\powersim2\examples"

capture log close
log using powersim2.log, text replace

global GraphWidth16x9 = 1200
global GraphHeight16x9 = 720

// IDENTIFY REASONABLE PARAMETER VALUES
webuse nhanes2, clear
//regress bpsystol age ib1.sex height weight
regress bpsystol c.age##ib1.sex 
logistic highbp c.age##ib1.sex 


set seed 15
clear
set obs 100
generate age = runiformint(18,65)
generate female = rbinomial(1,0.5)
generate interact = age*female
generate e = rnormal(0,20)

generate sbp = 110 + 0.5*age + (-20)*female + 0.35*interact  + e

regress sbp age i.female c.age#i.female
estimates store full
quietly regress sbp age i.female
estimates store reduced
lrtest full reduced 
return list
local reject = (r(p)<0.05)

display "reject = `reject'"




// BASIC -simregress- COMMAND
capture program drop simregress
program simregress, rclass
	version 15
	// PARSE INPUT
	syntax, n(integer)			/// 
		  [ alpha(real 0.05) 	///
			intercept(real 110)	///
			age(real 0.5) 		///
			female(real -20) 	///
			interact(real 0.35)	///
			esd(real 20) ]
	// COMPUTE POWER
	quietly {
		drop _all
		set obs `n'
		generate age = runiformint(18,65)
		generate female = rbinomial(1,0.5)
		generate interact = age*female
		generate e = rnormal(0,`esd')

		generate sbp = `intercept' + `age'*age + `female'*female + `interact'*interact  + e

		regress sbp age i.female c.age#i.female
		estimates store full
		regress sbp age i.female
		estimates store reduced
		lrtest full reduced
	}
	// RETURN RESULTS
	return scalar reject = (r(p)<`alpha')
end

simregress, n(100) age(0.5) female(20) interact(0.35) esd(20) alpha(0.05)



simulate reject=r(reject), reps(100): 	///
	simregress, n(100) age(0.5) female(20) interact(0.35) esd(20) alpha(0.05)

summarize reject
local power = r(mean)




capture program drop power_cmd_simregress
program power_cmd_simregress, rclass
	version 15
	// PARSE INPUT
	syntax, n(integer)			/// 
		  [	alpha(real 0.05) 	///
			intercept(real 110)	///
			age(real 0.5) 		///
			female(real -20) 	///
			interact(real 0.35)	///
			esd(real 20) 		///
			reps(integer 100) ]
	// COMPUTE POWER
	quietly {
		simulate reject=r(reject), reps(`reps'): 	    ///
		simregress, n(`n') age(`age') female(`female')  ///
		            interact(`interact') esd(`esd') alpha(`alpha')
		summarize reject
	}

	// RETURN RESULTS
	return scalar power = r(mean)
	return scalar N = `n'
	return scalar alpha = `alpha'
	return scalar intercept = `intercept'
	return scalar age = `age'
	return scalar female = `female'
	return scalar interact = `interact'
	return scalar esd = `esd'
end


// ADD PARAMETER FLEXIBILITY TO -power simregress-
// ==========================================================================
capture program drop power_cmd_simregress_init
program power_cmd_simregress_init, sclass
	sreturn local pss_colnames "intercept age female interact esd"
	sreturn local pss_numopts  "intercept age female interact esd"
end


power simregress, n(400(100)700) intercept(110) age(0.5) female(-20) interact(0.2(0.05)0.4) ///
                  reps(1000)                                                                ///
				  table(alpha power n intercept age female interact, separator(4))          ///
				  graph(xdimension(interact) legend(rows(1)))
graph export ./graphs/simregress2.png, as(png) width($GraphWidth16x9) height($GraphHeight16x9) replace




// =============================================================================
//                       -------    LOGISTIC   ---------
// =============================================================================

// ALL INPUT PARAMETERS ARE ON AN ODDS RATIO SCALE!!!

// CREATE LOGISTIC DATA AND FIT THE MODEL
set seed 123456
clear
set obs 100
generate age = runiformint(18,65)
generate female = rbinomial(1,0.5)
generate interact = age*female
generate xb = (ln(0.17) + ln(1.04)*age + ln(0.15)*female + ln(1.03)*interact)
generate highbp = rlogistic(xb,1) > 0

logistic highbp age i.female c.age#i.female



// BASIC -simlogit- COMMAND
capture program drop simlogit
program simlogit, rclass
	version 15
	// PARSE INPUT
	syntax, n(integer)				/// 
		  [ alpha(real 0.05) 		///
			intercept(real 0.17)	///
			age(real 1.04) 			///
			female(real 0.15) 		///
			interact(real 1.03) ]
	// COMPUTE POWER
	quietly {
		drop _all
		set obs `n'
		generate age = runiformint(18,65)
		generate female = rbinomial(1,0.5)
		generate interact = age*female
		generate xb = (ln(`intercept') + ln(`age')*age + ln(`female')*female + ln(`interact')*interact)
		generate highbp = rlogistic(xb,1) > 0

		logistic highbp age i.female c.age#i.female
		estimates store full
		logistic highbp age i.female
		estimates store reduced
		lrtest full reduced
	}
	// RETURN RESULTS
	return scalar reject = (r(p)<`alpha')
end

simlogit, n(500) intercept(0.17) age(1.04) female(.15) interact(1.03) alpha(0.05)

simulate reject=r(reject), reps(100): 	///
	simlogit, n(500) intercept(0.17) age(1.04) female(.15) interact(1.03) alpha(0.05)

summarize reject
local power = r(mean)




capture program drop power_cmd_simlogit
program power_cmd_simlogit, rclass
	version 15.1
	// PARSE INPUT
	syntax, n(integer)				/// 
		  [ alpha(real 0.05) 		///
			intercept(real 0.17)	///
			age(real 1.04) 			///
			female(real 0.15) 		///
			interact(real 1.03) 	///
			reps(integer 100)]
	// COMPUTE POWER
	quietly {
		simulate reject=r(reject), reps(`reps'): 	       ///
		simlogit, n(`n') intercept(`intercept') age(`age') ///
		          female(`female') interact(`interact') alpha(`alpha')
		summarize reject
		local power = r(mean)
	}
	// RETURN RESULTS
	return scalar power = r(mean)
	return scalar N = `n'
	return scalar alpha = `alpha'
	return scalar intercept = `intercept'
	return scalar age = `age'
	return scalar female = `female'
	return scalar interact = `interact'
end


// ADD PARAMETER FLEXIBILITY TO -power simregress-
// ==========================================================================
capture program drop power_cmd_simlogit_init
program power_cmd_simlogit_init, sclass
	sreturn local pss_colnames "intercept age female interact"
	sreturn local pss_numopts  "intercept age female interact"
end



power simlogit, n(500) intercept(0.17) age(1.04) female(.15) interact(1.03) alpha(0.05) reps(1000)

power simlogit, n(400(200)1000) intercept(0.17) age(1.04) female(.15) interact(1.02(0.01)1.05)  ///
                reps(1000) table graph(xdimension(interact) legend(rows(1)))
graph export ./graphs/simlogit.png, as(png) width($GraphWidth16x9) height($GraphHeight16x9) replace





log close




