

clear

capture cd "C:\Users\ChuckStata\Google Drive\Stata\blog\2018\powersim3\examples"
capture cd "C:\Users\jch\Google Drive\Stata\blog\2018\powersim3\examples"

capture log close
log using powersim3.log, text replace

set seed 15

global GraphWidth16x9 = 1920
global GraphHeight16x9 = 1080


// USE THE PARAMETERS FROM THIS MODEL TO CREATE THE DATASETS
use http://www.stata-press.com/data/r15/childweight.dta, clear
// Convert age (years) to agem (months)
generate agem = age*12
mixed weight c.age##i.girl || id: age, stddev






// SIMULATE LONGITUDINAL DATA
// =======================================================
set seed 15
clear
set obs 200
generate child = _n
generate female = rbinomial(1,0.5)
generate u_0i = rnormal(0,0.25)
generate u_1i = rnormal(0,0.60)
expand 5
bysort child: generate age = (_n-1)*0.5
generate interaction = age*female
generate e_ij = rnormal(0,1.2)
generate weight = 5.35 + 3.6*age + (-0.5)*female + (-0.25)*interaction  + u_0i + age*u_1i + e_ij
list child weight age female u_0i u_1i e_ij if child==1

mixed weight age i.female c.age#i.female || child: age , stddev nolog noheader
estimates store full
local parameter =_b[1.female#c.age]
mixed weight age i.female || child: age , stddev nolog noheader
estimates store reduced
lrtest full reduced
local reject = (r(p) < 0.05)


// DEAL WITH CONVERGENCE ISSUES
mixed weight age i.female c.age#i.female || child: age , stddev nolog noheader iter(500)
display e(converged)
scalar converged1 = r(converged)

if converged1 == 1 {
   display "The model converged"
}







capture program drop simmixed
program simmixed, rclass
	version 15
	// PARSE INPUT
	syntax, n1(integer)			/// 
			n(integer)			/// 
		  [ alpha(real 0.05) 		///
			intercept(real 5.35)	///
			age(real 3.6) 			///
			female(real -0.5) 		///
			interact(real -0.25)	///
			u0i(real 0.25)			/// 
			u1i(real 0.60) 			///
			eij(real 1.2) ]

	// COMPUTE POWER		
	quietly {		
		drop _all
		set obs `n'
		generate child = _n
		generate female = rbinomial(1,0.5)
		generate u_0i = rnormal(0,`u0i')
		generate u_1i = rnormal(0,`u1i')
		expand `n1'
		bysort child: generate age = (_n-1)*0.5
		generate interaction = age*female
		generate e_ij = rnormal(0,`eij')
		generate weight = `intercept' + `age'*age + `female'*female + `interact'*interaction  + u_0i + age*u_1i + e_ij

		mixed weight age i.female c.age#i.female || child: age , iter(500)
		estimates store full
		mixed weight age i.female || child: age , iter(500)
		estimates store reduced
		lrtest full reduced
	}
	// RETURN RESULTS
	    return scalar reject = (r(p)<`alpha')
end


simulate reject=r(reject), reps(10) seed(12345): 	///
	     simmixed, n1(5) n(1000)
summarize reject
local power = r(mean)







// DEFINE -power simmixed- COMMAND
capture program drop power_cmd_simmixed
program power_cmd_simmixed, rclass
	version 15
	// PARSE INPUT
	syntax, n1(integer)			/// 
			n(integer)			/// 
		  [ alpha(real 0.05) 		///
			intercept(real 5.35)	///
			age(real 3.6) 			///
			female(real -0.5) 		///
			interact(real -0.25)	///
			u0i(real 0.25)			/// 
			u1i(real 0.60) 			///
			eij(real 1.2) 			///
			reps(integer 1000) ]

	// COMPUTE POWER		
	quietly {
		simulate reject=r(reject), reps(`reps'):            				    ///
		simmixed, n1(`n1') n(`n') alpha(`alpha') intercept(`intercept') 	///
				  age(`age') female(`female') interact(`interact') 			    ///
				  u0i(`u0i') u1i(`u1i') eij(`eij')
		summarize reject
	}
	
	// RETURN RESULTS
	return scalar power = r(mean)
	return scalar n1 = `n1'
	return scalar N = `n'
	return scalar alpha = `alpha'
	return scalar intercept = `intercept'
	return scalar age = `age'
	return scalar female = `female'
	return scalar interact = `interact'
	return scalar u0i = `u0i'
	return scalar u1i = `u1i'
	return scalar eij = `eij'
end


// ADD PARAMETER FLEXIBILITY TO -power simregress-
// ==========================================================================
capture program drop power_cmd_simmixed_init
program power_cmd_simmixed_init, sclass
	version 15
	sreturn clear
	// ADD COLUMNS TO THE OUTPUT TABLE
	sreturn local pss_colnames "n1 intercept age female interact u0i u1i eij"
	// ALLOW NUMLISTS FOR ALL PARAMETERS
	sreturn local pss_numopts  "n1 intercept age female interact u0i u1i eij"
end

power simmixed, n1(5) n(100) reps(10000)

power simmixed, n1(5 6) n(100(100)500) reps(1000)	                        ///
	  table(n1 N power)						                                ///
	  graph(ydimension(power) xdimension(N) plotdimension(n1)               ///
	        xtitle(Level 2 Sample Size) legend(title(Level 1 Sample Size)))
	  
graph export ./graphs/simmixed.png, as(png) width($GraphWidth16x9) height($GraphHeight16x9) replace 




log close


